@@ -191,6 +191,29 @@ class OpenIDConnectAccessToken(loading.BaseFederationLoader):
         return options
 
 
+class OpenIDConnectDeviceAuthorization(_OpenIDConnectBase):
+
+    @property
+    def plugin_class(self):
+        return identity.V3OidcDeviceAuthorization
+
+    def get_options(self):
+        options = super(OpenIDConnectDeviceAuthorization, self).get_options()
+
+        # RFC 8628 doesn't support id_token
+        options = [opt for opt in options if opt.name != 'access-token-type']
+
+        options.extend([
+            loading.Opt('device-authorization-endpoint',
+                        help='OAuth 2.0 Device Authorization Endpoint. Note '
+                        'that if a discovery document is being passed this '
+                        'option will override the endpoint provided by the '
+                        'server in the discovery document.'),
+        ])
+
+        return options
+
+
 class TOTP(loading.BaseV3Loader):
 
     @property
@@ -338,3 +361,75 @@ class MultiFactor(loading.BaseV3Loader):
         self._methods = kwargs['auth_methods']
 
         return super(MultiFactor, self).load_from_options(**kwargs)
+
+
+class OAuth2ClientCredential(loading.BaseV3Loader):
+
+    @property
+    def plugin_class(self):
+        return identity.V3OAuth2ClientCredential
+
+    def get_options(self):
+        options = super(OAuth2ClientCredential, self).get_options()
+        options.extend([
+            loading.Opt('oauth2_endpoint',
+                        required=True,
+                        help='Endpoint for OAuth2.0'),
+        ]),
+        options.extend([
+            loading.Opt('oauth2_client_id',
+                        required=True,
+                        help='Client id for OAuth2.0'),
+        ]),
+        options.extend([
+            loading.Opt('oauth2_client_secret',
+                        secret=True,
+                        required=True,
+                        help='Client secret for OAuth2.0'),
+        ])
+
+        return options
+
+    def load_from_options(self, **kwargs):
+        if not kwargs.get('oauth2_endpoint'):
+            m = 'You must provide an OAuth2.0 endpoint.'
+            raise exceptions.OptionError(m)
+        if not kwargs.get('oauth2_client_id'):
+            m = 'You must provide an OAuth2.0 client credential ID.'
+            raise exceptions.OptionError(m)
+        if not kwargs.get('oauth2_client_secret'):
+            m = 'You must provide an OAuth2.0 client credential auth secret.'
+            raise exceptions.OptionError(m)
+
+        return super(OAuth2ClientCredential, self).load_from_options(**kwargs)
+
+
+class OAuth2mTlsClientCredential(loading.BaseV3Loader):
+
+    @property
+    def plugin_class(self):
+        return identity.V3OAuth2mTlsClientCredential
+
+    def get_options(self):
+        options = super(OAuth2mTlsClientCredential, self).get_options()
+        options.extend([
+            loading.Opt('oauth2-endpoint',
+                        required=True,
+                        help='Endpoint for OAuth2.0 Mutual-TLS Authorization'),
+            loading.Opt('oauth2-client-id',
+                        required=True,
+                        help='Client credential ID for OAuth2.0 Mutual-TLS '
+                             'Authorization')
+        ])
+        return options
+
+    def load_from_options(self, **kwargs):
+        if not kwargs.get('oauth2_endpoint'):
+            m = 'You must provide an OAuth2.0 Mutual-TLS endpoint.'
+            raise exceptions.OptionError(m)
+        if not kwargs.get('oauth2_client_id'):
+            m = ('You must provide an client credential ID for '
+                 'OAuth2.0 Mutual-TLS Authorization.')
+            raise exceptions.OptionError(m)
+        return super(OAuth2mTlsClientCredential,
+                     self).load_from_options(**kwargs)
